#!/usr/bin/env python3

import sys
from math import floor, log10
import asyncio


def aiorun(f):
    loop = asyncio.get_event_loop()
    loop.run_until_complete(f)
    loop.close()

asyncio.run = aiorun



async def main():
    if len(sys.argv) != 2 and len(sys.argv) != 3:
        help()
    else:
        if len(sys.argv) == 3:
            #asyncio.run(burn(float(sys.argv[1]), int(sys.argv[2])))
            await burn(float(sys.argv[1]), int(sys.argv[2]))
        else:
            #asyncio.run(burn(float(sys.argv[1]), 1))
            await burn(float(sys.argv[1]), 1)

def help():
    print("Single core burner computing fib(n) recursively.")
    print("  Usage: {} <hours> [number of threads]".format(sys.argv[0]))

async def burn(hours, threads):
    """
    Solve:
        time(fib(36)) = 10s
        time(fib(n)) = c * 1.618**n

        10 = c ** 1.618**n
        log10(10)/log10(1.618) = log10(c)/log10(1.618) + n
        log10(10) - n*log10(1.618) = log10(c)
        10**(1 - n*log10(1.618)) = c

    """
    # c is the characteristic speed of this cpu and I measured time(fib(36)) = 10s
    c = 10**(1 - 36*log10(1.618))
    n = log10(hours*3600/c)/log10(1.618)
    n = int(floor(n))
    print("Burning {} hours by stupidly calculating fib({}).".format(hours, n))
    #results = await asyncio.gather(*[fib(n) for i in range(threads)])
    results = await asyncio.gather(*[fib(n,i) for i in range(threads)])
    for r in results:
        print("fib({}) = {} , congrats.".format(n, r))



async def fib(n, id):
    if n == 0 or n == 1:
        return 1
    else:
        a = await fib(n-1, id)
        b = await fib(n-2, id)
        return a + b

if __name__ == '__main__':
    asyncio.run(main())
